#!/bin/sh
#
# Copyright 1997, Swiss Federal Institute of Technology, Zuerich (ETHZ)
#
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of ETHZ not be used in advertising or
# publicity pertaining to distribution of the software without specific,
# written prior permission.  ETHZ makes no representations about the
# suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
#
# ETHZ DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL ETHZ
# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#
#------------------------------------------------------------------------------
# Project:               Crossbow (Originated from Da CaPo++)
#------------------------------------------------------------------------------
#
# File:			$Source: /home/arl/arlsw/mwa/Admin/cvsroot/home/mwa/src/error/Config2h.sh,v $
# Author(s):		Marcel Waldvogel
# Affiliation:		ETH Zuerich, TIK
# Version:		$Revision: 1.4 $
# Creation Date:	2.7.97
# Last Changed:		$Date: 1997/08/10 18:59:08 $ by $Author: mwa $
#------------------------------------------------------------------------------
# 
# * Concept and Idea:
#	Gives the possibility for an abstract error definition file
#
# * Performed Functions:
#	Converts Error.config to ErrorConfig.h.
#
# * Usage:
#	config2h.sh [-v] <configfile> < <infile> > <outfile>
#		-v turns on verbose mode
#
#------------------------------------------------------------------------------
#
# * Solved Errors (Function, Type of Error, Solution, Author, Date):
# $Log: Config2h.sh,v $
# Revision 1.4  1997/08/10 18:59:08  mwa
# Now seems to work (tested in AtariImageFilter)
#
# Revision 1.3  1997/08/05 02:17:17  mwa
# Made a working version (preliminary)
#
# Revision 1.2  1997/07/09 23:58:15  mwa
# Current version checked in
#
# Revision 1.1  1997/07/09 08:36:02  mwa
# Initial revision
#
#------------------------------------------------------------------------------
#
# * Remarks:
# 
#==============================================================================

extract() {
	# Usage: extract <file> <nametag> <varname>
	# Extracts everything between the lines "<nametag> {" and "};".
	# It takes care of comments and gets rid of leading or trailing
	# whitespace.
	# It also gets rid of "illegal" chars, which may be misinterpreted
	# by the shell
	tr -s ' \011' '  ' < $1 |
	       sed -n -e 's/#.*//' -e 's/^ *//' -e 's/ *$//' \
		      -e '/^$/d' -e "/^$2"' {$/,/^};$/p' |
	       sed -e "/^$2"' {$/d' -e '/^};/d' |
	       tr -c '\012 0-9A-Za-z=' '_'
}

# Check for verbose flag
if [ "$1" = "-v" ]; then
	verbose=true
	shift
else
	verbose=false
fi

# Now we need to file names
if [ $# != 2 ]; then
	echo "$0: Bad usage, see source" >&2
	exit 1
fi

# Some shells forget args when calling functions: remember them
cfg=$1
filename=$2

# Get variables
mp=`extract $cfg "Macro prefix"`
ecp=`extract $cfg "Error class prefix"`
esep=`extract $cfg "Error severity enum prefix"`
esdp=`extract $cfg "Error severity define prefix"`
levels=`extract $cfg "Error levels"`
subsystems=`extract $cfg "Subsystems"`
deflevel=`extract $cfg "Default logging level"`
defname=`extract $cfg "Set-all name"`

linenum=1
echo "#line $linenum \"$filename\""
# Transform input to output
# All lines are put between hash signs to preserve leading/trailing whitespace
# Any line generated in here which starts or ends with a hash sign,
# has to have it doubled
sed -e 's/^/#/' -e 's/$/#/' -e 's/\\/\\\\\\\\/g' | (
while read line; do
linenum=`expr $linenum + 1`
# Create line number for "pseudo-include" file
case "$line" in
"##GENERATE"*"#")
	echo "##line 1 \"$filename/$line\""
	echo "/* START $line */"
	;;
esac
case "$line" in
# Make the ER_SEV_* #defines
"##GENERATE ER_SEV_DEFINES#")
	i=0
	for symbol in `echo "$levels" | tr 'a-z' 'A-Z'`; do
		echo "##define $esdp$symbol	$i"
		i=`expr $i + 1`
	done
	;;
# Give names to the ER_SEV_* #defines
"##GENERATE ER_SEV_NAMES#")
	for symbol in `echo "$levels" | tr 'a-z' 'A-Z'`; do
		echo "    \"$symbol\","
	done
	;;
# Make the ErrorSeverity
"##GENERATE ERRORSEVERITY#")
	for symbol in $levels; do
		SYMBOL=`echo "$symbol" | tr 'a-z' 'A-Z'`
		echo "    $esep$symbol	= $esdp$SYMBOL,"
	done
	;;
# Make the ErrorClass enum
"##GENERATE SUBSYSTEMS#")
	for symbol in $subsystems; do
		Symbol=`echo "$symbol" | sed 's/=.*//'`
		echo "    $ecp$Symbol,"
	done
	echo "    ${ecp}MaxClass,"
	;;
# Make the ErrorClass struct
"##GENERATE SUBSYSTEMDEFAULTS#")
	for symbol in $subsystems; do
		Symbol=`echo "$symbol" | sed 's/=.*//'`
		Value=`echo "$symbol" | sed -n -e 's/.*=//p' | tr 'a-z' 'A-Z'`
		if [ "$Value" = "" ]; then
			Value=`echo "$deflevel" | tr 'a-z' 'A-Z'`
		fi
		echo "    {\"$Symbol\",	$esdp$Value},"
	done
	echo "    {\"$defname\",	$esdp"`echo "$deflevel" | tr 'a-z' 'A-Z'`"},"
	;;
# Make the macros
"##GENERATE MACROS#")
	# The first one needs the "And" in the name, because it
	# logs _and_ aborts
	and="And"
	for symbol in $levels; do
		SYMBOL=`echo "$symbol" | tr 'a-z' 'A-Z'`
		echo "##define $prefix$SYMBOL if (DEBUG_LEVEL <= $esdp$SYMBOL) ErrorLog$and$symbol"
		echo "##define $prefix${SYMBOL}_XA if (DEBUG_LEVEL <= $esdp$SYMBOL) ErrorLog$and${symbol}Xa"
		# All further entries don't have the "And"
		and=""
	done
	;;
# Make the prototypes
"##GENERATE PROTOTYPES#")
	# The first one needs the "And" in the name, because it
	# logs _and_ aborts
	and="And"
	noret=", noreturn"
	for symbol in $levels; do
		echo "void"
		echo "ErrorLog$and$symbol(ErrorClass errorClass, ErrorString name,"
		echo "		ErrorString format, ...)"
		echo "		__attribute__((format(printf, 3, 4)$noret));"
		echo "void"
		echo "ErrorLog$and${symbol}Xa(ErrorClass errorClass, ErrorString name,"
		echo "		ErrorExtendedLogFunc moreInfo, void *moreData,"
		echo "		ErrorString format, ...)"
		echo "		__attribute__((format(printf, 5, 6)$noret));"
		# All further entries don't have the "And"
		and=""
		noret=""
	done
	;;
# Make the functions
"##GENERATE FUNCTIONS#")
	# The first one needs the "And" in the name, because it
	# logs _and_ aborts
	and="And"
	for symbol in $levels; do
		SYMBOL=`echo "$symbol" | tr 'a-z' 'A-Z'`
		echo "void"
		echo "ErrorLog$and$symbol(ErrorClass errorClass, ErrorString name,"
		echo "		ErrorString format, ...)"
		echo "{"
		echo "    va_list ap;"
		echo "    va_start(ap, format);"
		echo "    ErrorVLogMessage(errorClass, name, $esdp$SYMBOL, NULL, NULL, format, ap);"
		echo "    va_end(ap);"
		[ -z "$and" ] || echo "    ErrorAbort(1, 0);"
		echo "}"

		echo "void"
		echo "ErrorLog$and${symbol}Xa(ErrorClass errorClass, ErrorString name,"
		echo "		ErrorExtendedLogFunc moreInfo, void *moreData,"
		echo "		ErrorString format, ...)"
		echo "{"
		echo "    va_list ap;"
		echo "    va_start(ap, format);"
		echo "    ErrorVLogMessage(errorClass, name, $esdp$SYMBOL,"
		echo "                     moreInfo, moreData, format, ap);"
		echo "    va_end(ap);"
		[ -z "$and" ] || echo "    ErrorAbort(1, 0);"
		echo "}"
		# All further entries don't have the "And" and don't abort
		and=""
	done
	;;
*)
	echo "$line"
	;;
esac
# Create "return" line number from "pseudo-include" file
case "$line" in
"##GENERATE"*"#")
	echo "##line $linenum \"$filename\""
	echo "/* END $line */"
	;;
esac
done
) | sed -e 's/^#//' -e 's/#$//'

exit 0
